@@ -18,6 +18,7 @@ import logging
 import textwrap
 import sys
 
+
 def true_long_type():
     """
     OS X uses an 8-byte long, so make sure L (long) is the right size
@@ -26,11 +27,13 @@ def true_long_type():
     for type_ in ['L', 'I']:
         test_array = array(type_, [0])
         long_size = test_array.itemsize
-        if long_size == 4: return type_
+        if long_size == 4:
+            return type_
     raise ImportError("Couldn't determine a valid 4-byte long type to use \
 as equivalent to LONG")
 LONG = true_long_type()
 
+
 def byte_to_bases(x):
     """convert one byte to the four bases it encodes"""
     c = (x >> 4) & 0xf
@@ -41,22 +44,37 @@ def byte_to_bases(x):
     ff = f & 0x3
     return map(bits_to_base, (cc, cf, fc, ff))
 
+
 def bits_to_base(x):
     """convert integer representation of two bits to correct base"""
-    if x is 0: return 'T'
-    if x is 1: return 'C'
-    if x is 2: return 'A'
-    if x is 3: return 'G'
+    if x is 0:
+        return 'T'
+    elif x is 1:
+        return 'C'
+    elif x is 2:
+        return 'A'
+    elif x is 3:
+        return 'G'
+    else:
+        raise ValueError('Only integers 0-3 are valid inputs')
+
 
 def base_to_bin(x):
     """
     provided for user convenience
-    convert a nucleotide to its bit representation 
+    convert a nucleotide to its bit representation
     """
-    if x == 'T': return '00'
-    if x == 'C': return '01'
-    if x == 'A': return '10'
-    if x == 'G': return '11'
+    if x == 'T':
+        return '00'
+    elif x == 'C':
+        return '01'
+    elif x == 'A':
+        return '10'
+    elif x == 'G':
+        return '11'
+    else:
+        raise ValueError('Only characters \'ATGC\' are valid inputs')
+
 
 def create_byte_table():
     """create BYTE_TABLE"""
@@ -65,6 +83,7 @@ def create_byte_table():
         d[x] = byte_to_bases(x)
     return d
 
+
 def split16(x):
     """
     split a 16-bit number into integer representation
@@ -74,6 +93,7 @@ def split16(x):
     f = x & 0xff
     return c, f
 
+
 def create_twobyte_table():
     """create TWOBYTE_TABLE"""
     d = {}
@@ -85,45 +105,79 @@ def create_twobyte_table():
 BYTE_TABLE = create_byte_table()
 TWOBYTE_TABLE = create_twobyte_table()
 
+
 def longs_to_char_array(longs, first_base_offset, last_base_offset, array_size,
                         more_bytes=None):
+    if more_bytes is not None: print array('B', more_bytes)
     """
-    takes in a iterable of longs and converts them to bases in a char array
-    returns a ctypes string buffer
+    takes in an array of longs (4 bytes) and converts them to bases in
+    a char array
+    you must also provide the offset in the first and last block
+    (note these offsets are pythonic. last_offset is not included)
+    and the desired array_size
+    If you have less than a long worth of bases at the end, you can provide
+    them as a string with more_bytes=
+    
+    NOTE: last_base_offset is inside more_bytes not the last long, if more_bytes
+          is not None
+    returns the correct subset of the array based on provided offsets
     """
+    if array_size == 0:
+        return array('c')
+    elif array_size < 0:
+        raise ValueError('array_size must be at least 0')
+
+    if not first_base_offset in range(16):
+        raise ValueError('first_base_offset must be in range(16)')
+    if not last_base_offset in range(1, 17):
+        raise ValueError('last_base_offset must be in range(1, 17)')
+
     longs_len = len(longs)
-    # dna = ctypes.create_string_buffer(array_size)
-    dna = array('c', 'N' * longs_len)
+    if more_bytes is None:
+        shorts_length = 0
+    else:
+        shorts_length = len(more_bytes)
+    if array_size > longs_len * 16 + 4 * shorts_length:
+        raise ValueError('array_size exceeds maximum possible for input')
+
+    dna = array('c', 'N' * (longs_len * 16 + 4 * shorts_length))
     # translate from 32-bit blocks to bytes
     # this method ensures correct endianess (byteswap as neeed)
-    bytes = array('B')
-    bytes.fromstring(longs.tostring())
+    bytes_ = array('B')
+    bytes_.fromstring(longs.tostring())
     # first block
-    first_block = ''.join([''.join(BYTE_TABLE[bytes[x]]) for x in range(4)])
+    first_block = ''.join([''.join(BYTE_TABLE[bytes_[x]]) for x in range(4)])
     i = 16 - first_base_offset
-    if array_size < i: i = array_size
+    if array_size < i:
+        i = array_size
     dna[0:i] = array('c', first_block[first_base_offset:first_base_offset + i])
     if longs_len > 1:
         # middle blocks (implicitly skipped if they don't exist)
-        for byte in bytes[4:-4]:
+        for byte in bytes_[4:-4]:
             dna[i:i + 4] = array('c', BYTE_TABLE[byte])
             i += 4
         # last block
-        last_block = array('c', ''.join([''.join(BYTE_TABLE[bytes[x]]) for x in range(-4,0)]))
-        dna[i:i + last_base_offset] = last_block[0:last_base_offset]
+        last_block = array('c', ''.join([''.join(BYTE_TABLE[bytes_[x]])
+                                         for x in range(-4, 0)]))
+        if more_bytes is None:
+            dna[i:i + last_base_offset] = last_block[0:last_base_offset]
+        else: # if there are more bytes, we need the whole last block
+            dna[i:i + 16] = last_block[0:16]
         i += 16
     if more_bytes is not None:
-        bytes = array('B')
-        bytes.fromstring(more_bytes)
+        bytes_ = array('B')
+        bytes_.fromstring(more_bytes)
         j = i
-        for byte in bytes:
+        for byte in bytes_:
             j = i + 4
             if j > array_size:
-                dna[i:array_size] = array('c', BYTE_TABLE[byte])[0:(array_size - i)]
+                dnabytes = array('c', BYTE_TABLE[byte])[0:(array_size - i)]
+                dna[i:array_size] = dnabytes
                 break
-            dna[i:i + 4] = array('c', BYTE_TABLE[byte])
+            dna[i:i + last_base_offset] = array('c', BYTE_TABLE[byte])
             i += 4
-    return dna
+    return dna[0:array_size]
+
 
 class TwoBitFile(dict):
     """
@@ -138,16 +192,16 @@ You may access sequences by name, e.g.
 Sequences are returned as TwoBitSequence objects
 You may access intervals by slicing or using str() to dump the entire entry
 e.g.
->>> chr20[100100:100200]
-'ttttcctctaagataatttttgccttaaatactattttgttcaatactaagaagtaagataacttccttttgttggtat
-ttgcatgttaagtttttttcc'
+>>> chr20[100100:100120]
+'ttttcctctaagataatttttgccttaaatactattttgttcaatactaagaagtaagataacttccttttgttggta
+tttgcatgttaagtttttttcc'
 >>> whole_chr20 = str(chr20)
 
 Fair warning: dumping the entire chromosome requires a lot of memory
 
 See TwoBitSequence for more info
     """
-    
+
     def __init__(self, foo):
         super(TwoBitFile, self).__init__()
         if not exists(foo):
@@ -163,8 +217,8 @@ See TwoBitSequence for more info
             self[name] = TwoBitSequence(self._file_handle, offset,
                                         self._file_size,
                                         self._byteswapped)
-        return        
-        
+        return
+
     def _load_header(self):
         file_handle = self._file_handle
         header = array(LONG)
@@ -178,15 +232,16 @@ See TwoBitSequence for more info
             header.byteswap()
             (signature2, version, sequence_count, reserved) = header
             if not signature2 == 0x1A412743:
-                raise TwoBitFileError('Signature in header should be 0x1A412743'
-                                    + ', instead found 0x%X' % signature)
-        if not version == 0: 
+                raise TwoBitFileError('Signature in header should be ' +
+                                      '0x1A412743, instead found 0x%X' %
+                                      signature)
+        if not version == 0:
             raise TwoBitFileError('File version in header should be 0.')
         if not reserved == 0:
             raise TwoBitFileError('Reserved field in header should be 0.')
         self._byteswapped = byteswapped
         self._sequence_count = sequence_count
-        
+
     def _load_index(self):
         file_handle = self._file_handle
         byteswapped = self._byteswapped
@@ -196,13 +251,16 @@ See TwoBitSequence for more info
         while remaining > 0:
             name_size = array('B')
             name_size.fromfile(file_handle, 1)
-            if byteswapped: name_size.byteswap()
+            if byteswapped:
+                name_size.byteswap()
             name = array('c')
             name.fromfile(file_handle, name_size[0])
-            if byteswapped: name.byteswap()
+            if byteswapped:
+                name.byteswap()
             offset = array(LONG)
             offset.fromfile(file_handle, 1)
-            if byteswapped: offset.byteswap()
+            if byteswapped:
+                offset.byteswap()
             sequence_offsets.append((name.tostring(), offset[0]))
             remaining -= 1
         self._sequence_offsets = sequence_offsets
@@ -217,10 +275,12 @@ See TwoBitSequence for more info
             file_handle.seek(offset)
             dna_size = array(LONG)
             dna_size.fromfile(file_handle, 1)
-            if byteswapped: dna_size.byteswap()
+            if byteswapped:
+                dna_size.byteswap()
             d[name] = dna_size[0]
         return d
 
+
 class TwoBitSequence(object):
     """
 A TwoBitSequence object refers to an entry in a TwoBitFile
@@ -230,12 +290,12 @@ e.g.
 >>> genome = TwoBitFile('hg18.2bit')
 >>> chr20 = genome['chr20']
 >>> chr20[100100:100200] # slicing returns a string
-'ttttcctctaagataatttttgccttaaatactattttgttcaatactaagaagtaagataacttccttttgttggtat
-ttgcatgttaagtttttttcc'
+'ttttcctctaagataatttttgccttaaatactattttgttcaatactaagaagtaagataacttccttttgttggta
+tttgcatgttaagtttttttcc'
 >>> whole_chr20 = str(chr20) # get whole chr as string
 
 Fair warning: dumping the entire chromosome requires a lot of memory
- 
+
 Note that we follow python/UCSC conventions:
 Coordinates are 0-based, end-open
 (Note: The UCSC web-based genome browser uses 1-based closed coordinates)
@@ -257,29 +317,36 @@ for k,v in d.iteritems(): d[k] = str(v)
         file_handle.seek(offset)
         header = array(LONG)
         header.fromfile(file_handle, 2)
-        if byteswapped: header.byteswap()
+        if byteswapped:
+            header.byteswap()
         dna_size, n_block_count = header
-        self._dna_size = dna_size # number of characters, 2 bits each
-        self._n_bytes = (dna_size + 3) / 4 # number of bytes
-        self._packed_dna_size = (dna_size + 15) / 16 # number of 32-bit fragments
+        self._dna_size = dna_size  # number of characters, 2 bits each
+        self._n_bytes = (dna_size + 3) / 4  # number of bytes
+        # number of 32-bit fragments
+        self._packed_dna_size = (dna_size + 15) / 16
         n_block_starts = array(LONG)
         n_block_sizes = array(LONG)
         n_block_starts.fromfile(file_handle, n_block_count)
-        if byteswapped: n_block_starts.byteswap()
+        if byteswapped:
+            n_block_starts.byteswap()
         n_block_sizes.fromfile(file_handle, n_block_count)
-        if byteswapped: n_block_sizes.byteswap()
+        if byteswapped:
+            n_block_sizes.byteswap()
         self._n_block_starts = n_block_starts
-        self._n_block_sizes= n_block_sizes
+        self._n_block_sizes = n_block_sizes
         mask_rawc = array(LONG)
         mask_rawc.fromfile(file_handle, 1)
-        if byteswapped: mask_rawc.byteswap()
+        if byteswapped:
+            mask_rawc.byteswap()
         mask_block_count = mask_rawc[0]
         mask_block_starts = array(LONG)
         mask_block_starts.fromfile(file_handle, mask_block_count)
-        if byteswapped: mask_block_starts.byteswap()
+        if byteswapped:
+            mask_block_starts.byteswap()
         mask_block_sizes = array(LONG)
         mask_block_sizes.fromfile(file_handle, mask_block_count)
-        if byteswapped: mask_block_sizes.byteswap()
+        if byteswapped:
+            mask_block_sizes.byteswap()
         self._mask_block_starts = mask_block_starts
         self._mask_block_sizes = mask_block_sizes
         file_handle.read(4)
@@ -298,16 +365,21 @@ for k,v in d.iteritems(): d[k] = str(v)
         # handle negative coordinates
         dna_size = self._dna_size
         if max_ is not None and max_ < 0:
-            if max_ < -dna_size: raise IndexError('index out of range')
+            if max_ < -dna_size:
+                raise IndexError('index out of range')
             max_ = dna_size + 1 + max_
         if min_ < 0:
-            if max_ < -dna_size: raise IndexError('index out of range')
+            if max_ < -dna_size:
+                raise IndexError('index out of range')
             min_ = dna_size + 1 + min_
         # make sure there's a proper range
-        if max_ is not None and min_ > max_: return ''
-        if max_ == 0 or max_ == min_: return ''
+        if max_ is not None and min_ > max_:
+            return ''
+        if max_ == 0 or max_ == min_:
+            return ''
         # load all the data
-        if max_ is None or max_ > dna_size: max_ = dna_size
+        if max_ is None or max_ > dna_size:
+            max_ = dna_size
         file_handle = self._file_handle
         byteswapped = self._byteswapped
         n_block_starts = self._n_block_starts
@@ -316,12 +388,14 @@ for k,v in d.iteritems(): d[k] = str(v)
         mask_block_sizes = self._mask_block_sizes
         offset = self._offset
         packed_dna_size = self._packed_dna_size
-        n_bytes = self._n_bytes
+        #n_bytes = self._n_bytes
+
+        # region_size is how many bases the region is
+        if max_ is None:
+            region_size = dna_size - min_
+        else:
+            region_size = max_ - min_
 
-        # region_size is how many bases the region is       
-        if max_ is None: region_size = dna_size - min_
-        else: region_size = max_ - min_
-        
         # start_block, end_block are the first/last 32-bit blocks we need
         # blocks start at 0
         start_block = min_ / 16
@@ -329,69 +403,84 @@ for k,v in d.iteritems(): d[k] = str(v)
         local_offset = offset + (start_block * 4)
         end_block = (max_ - 1 + 16) / 16
         # don't read past seq end
-        
+
         file_handle.seek(local_offset)
-        
+
         # note we won't actually read the last base
         # this is a python slice first_base_offset:16*blocks+last_base_offset
         first_base_offset = min_ % 16
         last_base_offset = max_ % 16
-        if last_base_offset == 0: last_base_offset = 16
+        if last_base_offset == 0:
+            last_base_offset = 16
         # +1 we still need to read end_block maybe
         blocks_to_read = end_block - start_block
         if (blocks_to_read + start_block) > packed_dna_size:
             blocks_to_read = packed_dna_size - start_block
-        
+
         fourbyte_dna = array(LONG)
-        remainder_seq = None
+        # remainder_seq = None
         if (blocks_to_read * 4 + local_offset) > self._file_size:
             fourbyte_dna.fromfile(file_handle, blocks_to_read - 1)
-            morebytes = file_handle.read() # read the remaining characters
+            morebytes = file_handle.read()  # read the remaining characters
 #            if byteswapped:
 #                morebytes = ''.join(reversed(morebytes))
         else:
             fourbyte_dna.fromfile(file_handle, blocks_to_read)
             morebytes = None
-        if byteswapped: fourbyte_dna.byteswap()
-        string_as_array = longs_to_char_array(fourbyte_dna, first_base_offset,
-                                              last_base_offset, region_size,
-                                              more_bytes=morebytes)
+        if byteswapped:
+            fourbyte_dna.byteswap()
+        str_as_array = longs_to_char_array(fourbyte_dna, first_base_offset,
+                                           last_base_offset, region_size,
+                                           more_bytes=morebytes)
         for start, size in izip(n_block_starts, n_block_sizes):
             end = start + size
-            if end <= min_: continue
-            if start > max_: break
-            if start < min_: start = min_
-            if end > max_: end = max_ 
+            if end <= min_:
+                continue
+            if start > max_:
+                break
+            if start < min_:
+                start = min_
+            if end > max_:
+                end = max_
             start -= min_
             end -= min_
             # this should actually be decoded, 00=N, 01=n
-            string_as_array[start:end] = array('c', 'N'*(end-start))
+            str_as_array[start:end] = array('c', 'N'*(end-start))
         lower = str.lower
         first_masked_region = max(0,
                                   bisect_right(mask_block_starts, min_) - 1)
         last_masked_region = min(len(mask_block_starts),
                                  1 + bisect_right(mask_block_starts, max_,
                                                   lo=first_masked_region))
-        for start, size in izip(mask_block_starts[first_masked_region:last_masked_region],
-                                mask_block_sizes[first_masked_region:last_masked_region]):
+        for start, size in izip(mask_block_starts[first_masked_region:
+                                                  last_masked_region],
+                                mask_block_sizes[first_masked_region:
+                                                 last_masked_region]):
             end = start + size
-            if end <= min_: continue
-            if start > max_: break
-            if start < min_: start = min_
-            if end > max_: end = max_ 
+            if end <= min_:
+                continue
+            if start > max_:
+                break
+            if start < min_:
+                start = min_
+            if end > max_:
+                end = max_
             start -= min_
             end -= min_
-            string_as_array[start:end] = array('c', lower(string_as_array[start:end].tostring()))
-        if not len(string_as_array) == max_ - min_:
-            raise RuntimeError, "Sequence was the wrong size"
-        return string_as_array.tostring()
+            str_as_array[start:end] = array('c',
+                                            lower(str_as_array[start:end].
+                                            tostring()))
+        if not len(str_as_array) == max_ - min_:
+            raise RuntimeError("Sequence was the wrong size")
+        return str_as_array.tostring()
 
     def __str__(self):
         """
         returns the entire chromosome
         """
         return self.__getslice__(0, None)
-    
+
+
 class TwoBitFileError(StandardError):
     """
     Base exception for TwoBit module
@@ -400,6 +489,7 @@ class TwoBitFileError(StandardError):
         errtext = 'Invalid 2-bit file. ' + msg
         return super(TwoBitFileError, self).__init__(errtext)
 
+
 def print_specification():
     """
     Prints the twoBit file format specification I got from the Internet.
@@ -445,7 +535,7 @@ nBlockSizes - the size of each block of N's
 maskBlockCount - the number of masked (lower case) blocks
 maskBlockStarts - starting position for each masked block
 maskBlockSizes - the size of each masked block
-packedDna - the dna packed to two bits per base as so: 00 - T, 01 - C, 10 - A, \
+packedDna - the dna packed to two bits per base as so: 00 - T, 01 - C, 10 - A,\
 11 - G. The first base is in the most significant 2 bits byte, and the last \
 base in the least significant 2 bits, so that the sequence TCAG would be \
 represented as 00011011. The packedDna field will be padded with 0 bits as \
@@ -455,6 +545,7 @@ improves i/o performance on some machines.
 .nib files
 """
 
+
 def cmdline_reader():
     """
     cmdline_reader allows twobitreader module to be executed as a script
@@ -462,13 +553,13 @@ def cmdline_reader():
     reads input (BED format) from stdin
     writes output (FASTA format) to stdout
     writes errors/warning to stderr
-    
+
     Regions should be given in BED format on stdin
     chrom    start(0-based)    end(0-based, not included)
-    
+
     To use a BED file of regions, do
     python -m twobitreader example.2bit < example.bed
-    
+
     Non-regions will be skipped and warnings will be issued to logging
     (logging output to stderr by default)
     """
@@ -492,44 +583,47 @@ def cmdline_reader():
     # otherwise proceed with opening the .2bit file
     twobit_file = TwoBitFile(argv[1])
     # print error/warning messages as we go
-    warning_msg = 'Invalid %s at line %d\n\t"%s"'
     twobit_reader(twobit_file, input_stream=sys.stdin)
 
+
 def twobit_reader(twobit_file, input_stream=None, write=None):
     """
     twobit_reader takes a twobit_file (of class TwoBitFile)
     and an "input_stream" which can be any iterable (incl. file-like objects)
-    writes output (FASTA format) using write (default is to print, if write=None)
+    writes output (FASTA format) using write (print if write=None)
     logs errors/warning to stderr
-    
+
     Regions should be given in BED format on stdin
     chrom    start(0-based)    end(0-based, not included)
-    
+
     To use a BED file of regions, do
     python -m twobitreader example.2bit < example.bed
-    
+
     Non-regions will be skipped and warnings will be issued to logging
     (logging output to stderr by default)
     """
     warning_msg = 'Invalid %s at line %d\n\t"%s"'
-    if input_stream is None: return
+    if input_stream is None:
+        return
     for i, line in enumerate((line.rstrip('\n\r') for line in input_stream)):
         fields = line.split()
         if not len(fields) >= 3:
             logging.warn(warning_msg, 'start', i, line)
             continue
         chrom = fields[0]
-        if not twobit_file.has_key(chrom):
+        if not chrom in twobit_file:
             logging.warn(warning_msg, 'chrom', i, line)
             continue
-        try: start = long(fields[1])
+        try:
+            start = long(fields[1])
         except ValueError:
             logging.warn(warning_msg, 'start', i, line)
         if start < 0:
             logging.warn(warning_msg, 'start', i, line)
             logging.warn('Using 0 as start instead for line %d', i)
             start = 0
-        try: end = long(fields[2])
+        try:
+            end = long(fields[2])
         except ValueError:
             logging.warn(warning_msg, 'end', i, line)
             continue
@@ -549,4 +643,5 @@ def twobit_reader(twobit_file, input_stream=None, write=None):
             print textwrap.fill(seq, 60)
     return
 
-if __name__ == '__main__': cmdline_reader()
\ No newline at end of file
+if __name__ == '__main__':
+    cmdline_reader()
